
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "Border.h"

using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::CalcItems;
using namespace AutomateIT::OpenOfficeWrapper;

CBorder::CBorder() : 
    m_borderIndex(xlEdgeTop),
    m_pParentTableBorder(NULL),
    m_pParentRange(NULL),
    m_pParentBorders(NULL)
{
    TRACINGMETHOD;
}

CBorder::~CBorder()
{
    TRACINGMETHOD;
}

HRESULT CBorder::Initialize( CBorders* pParentBorders, COORange* pRange, COOTableBorder* pTableBorder, XlBordersIndex borderIndex )
{
    TRACINGMETHOD;
    m_pParentBorders = pParentBorders;
    m_pParentTableBorder = pTableBorder;
    m_pParentRange = pRange;
    m_borderIndex = borderIndex;
    if (m_borderIndex == xlDiagonalUp || m_borderIndex == xlDiagonalDown)
    {
        m_borderIndex = xlEdgeTop;
    }

    COOBorderLinePtr pBorderLine(NULL);

    switch(m_borderIndex)
    {
    case xlEdgeTop:
        {
            hr = m_pParentTableBorder->get_TopLine(&pBorderLine);
            CHECKERROR;
        }
        break;
    case xlEdgeLeft:
        {
            hr = m_pParentTableBorder->get_LeftLine(&pBorderLine);
            CHECKERROR;
        }
        break;
    case xlEdgeRight:
        {
            hr = m_pParentTableBorder->get_RightLine(&pBorderLine);
            CHECKERROR;
        }
        break;
    case xlEdgeBottom:
        {
            hr = m_pParentTableBorder->get_BottomLine(&pBorderLine);
            CHECKERROR;
        }
        break;
    case xlInsideHorizontal:
        {
            hr = m_pParentTableBorder->get_HorizontalLine(&pBorderLine);
            CHECKERROR;
        }
        break;
    case xlInsideVertical:
        {
            hr = m_pParentTableBorder->get_VerticalLine(&pBorderLine);
            CHECKERROR;
        }
        break;
    }

    m_pBorderLine = pBorderLine;
    return hr;
}


HRESULT CBorder::SetChanges()
{
    TRACINGMETHOD;
    switch(m_borderIndex)
    {
    case xlEdgeTop:
        {
            hr = m_pParentTableBorder->put_TopLine(m_pBorderLine);
            CHECKERROR;
        }
        break;
    case xlEdgeLeft:
        {
            hr = m_pParentTableBorder->put_LeftLine(m_pBorderLine);
            CHECKERROR;
        }
        break;
    case xlEdgeRight:
        {
            hr = m_pParentTableBorder->put_RightLine(m_pBorderLine);
            CHECKERROR;
        }
        break;
    case xlEdgeBottom:
        {
            hr = m_pParentTableBorder->put_BottomLine(m_pBorderLine);
            CHECKERROR;
        }
        break;
    case xlInsideHorizontal:
        {
            hr = m_pParentTableBorder->put_HorizontalLine(m_pBorderLine);
            CHECKERROR;
        }
        break;
    case xlInsideVertical:
        {
            hr = m_pParentTableBorder->put_VerticalLine(m_pBorderLine);
            CHECKERROR;
        }
        break;
    }

    hr = m_pParentRange->put_TableBorder(m_pParentTableBorder);
    CHECKERROR;
    return hr;
}

HRESULT __stdcall CBorder::get_Application(_Application	**RHS)
{
    TRACINGMETHOD;
    return m_pParentBorders->get_Application(RHS);
}

HRESULT __stdcall CBorder::get_Creator(XlCreator *RHS)
{
    TRACINGMETHOD;
    (*RHS) = xlCreatorCode;
    return hr;
}

HRESULT __stdcall CBorder::get_Parent(IDispatch **RHS)
{
	TRACINGMETHOD;
    (*RHS) = m_pParentBorders;
    (*RHS)->AddRef();
	return hr;
}

HRESULT __stdcall CBorder::get_Color(VARIANT *RHS)
{
	TRACINGMETHOD;
    int color(0);
    hr = m_pBorderLine->get_Color(&color);
    CHECKERROR;
    (*RHS) = CComVariant(CColorPalette::ConvertRGB(color));
	return hr;
}

HRESULT __stdcall CBorder::put_Color(VARIANT RHS)
{
	TRACINGMETHOD;
    CComVariant color(RHS);
    hr = color.ChangeType(VT_INT);
    CHECKERROR;
    hr = m_pBorderLine->put_Color(CColorPalette::ConvertRGB(color.intVal));
    CHECKERROR;
    SetChanges();
    return hr;
}

HRESULT __stdcall CBorder::get_ColorIndex(VARIANT *RHS)
{
    TRACINGMETHOD;
    int color(0);
    hr = m_pBorderLine->get_Color(&color);
    CHECKERROR;
    (*RHS) = CComVariant(CColorPalette::GetColorIndex(color) + 1);
    return hr;
}

HRESULT __stdcall CBorder::put_ColorIndex(VARIANT RHS)
{
    TRACINGMETHOD; 
    CComVariant colorIndex(RHS);
    hr = colorIndex.ChangeType(VT_I4);
    CHECKERROR;
    if (colorIndex.intVal == xlColorIndexNone) 
    {
        colorIndex.intVal = 2;
    }
    if (colorIndex.intVal == xlColorIndexAutomatic) 
    {
        colorIndex.intVal = 1;
    }
    if (colorIndex.intVal == 0) 
    {
        colorIndex.intVal = 2;
    }

    hr = put_Color(CComVariant(CColorPalette::GetColor(colorIndex.intVal - 1)));
    return hr;
}

HRESULT __stdcall CBorder::get_LineStyle(VARIANT *RHS)
{
	TRACINGMETHOD;

    long inner(0);
    hr = m_pBorderLine->get_InnerLineWidth(&inner);
    CHECKERROR;

    long outer(0);
    hr = m_pBorderLine->get_OuterLineWidth(&outer);
    CHECKERROR;

    long distance(0);
    hr = m_pBorderLine->get_LineDistance(&distance);
    CHECKERROR;

    if ((0 == inner) && (0 == outer) && (0 == distance)) 
    {
        (*RHS) = CComVariant(xlLineStyleNone);
    }
    else if ((0 == distance)&&(0 == outer))
    {
        (*RHS) = CComVariant(xlContinuous);
    }
    else 
    {
        (*RHS) = CComVariant(xlDouble);
    }

    return hr;
}

HRESULT __stdcall CBorder::put_LineStyle(VARIANT RHS)
{
	TRACINGMETHOD;

    long inner(0);
    long outer(0);
    long distance(0);

    CComVariant lineStyle(RHS);
    hr = lineStyle.ChangeType(VT_I4);

    switch(lineStyle.intVal) 
    {
    case xlContinuous:
        {
            inner = 10;
            outer = 0;
            distance = 0;
        }
        break;
    case xlDouble:
        {
            inner = 10;
            outer = 10;
            distance = 10;
        }
        break;
    }

    hr = m_pBorderLine->put_InnerLineWidth(inner);
    CHECKERROR;
    hr = m_pBorderLine->put_OuterLineWidth(outer);
    CHECKERROR;
    hr = m_pBorderLine->put_LineDistance(distance);
    CHECKERROR;

    SetChanges();
    
    return hr;
}

HRESULT __stdcall CBorder::get_Weight( VARIANT *RHS )
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    // stub
    *RHS = CComVariant(1);
    return S_OK;
}

HRESULT __stdcall CBorder::put_Weight( VARIANT RHS )
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    // stub
    return S_OK;
}

