
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "Borders.h"

using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::OpenOfficeWrapper;

const XlBordersIndex CBorders::m_indices[] = 
{
    xlEdgeTop, 
    xlEdgeRight, 
    xlEdgeLeft, 
    xlEdgeBottom, 
    xlDiagonalUp, 
    xlDiagonalDown, 
    xlInsideVertical, 
    xlInsideHorizontal
}; 
const int CBorders::m_indicesSize = sizeof(CBorders::m_indices) / sizeof (XlBordersIndex);


CBorders::CBorders() : 
    m_pOOParentRange(NULL)
{
    TRACINGMETHOD;
}

CBorders::~CBorders()
{
    TRACINGMETHOD;
}

HRESULT CBorders::Initialize( CRange* pParentRange, COORange* pOORange )
{
    TRACINGMETHOD;
    m_pParentRange = pParentRange;
    m_pOOParentRange = pOORange;
    COOTableBorderPtr pTableBorder(NULL);
    hr = m_pOOParentRange->get_TableBorder(&pTableBorder);
    CHECKERROR;
    m_pTableBorder = pTableBorder;
    return hr;
}

HRESULT __stdcall CBorders::get_Application(_Application	**RHS)
{
	TRACINGMETHOD;
    return m_pParentRange->get_Application(RHS);
}

HRESULT __stdcall CBorders::get_Creator(XlCreator *RHS)
{
    TRACINGMETHOD;
    (*RHS) = xlCreatorCode;
    return hr;
}

HRESULT __stdcall CBorders::get_Parent(IDispatch **RHS)
{
    TRACINGMETHOD;
    (*RHS) = m_pParentRange;
    (*RHS)->AddRef();
    return hr;
}

HRESULT __stdcall CBorders::get_Color(VARIANT *RHS)
{
    TRACINGMETHOD;
    CComPtr<Border> pBorder(NULL);
    hr = get_Item(xlEdgeTop, &pBorder);
    CHECKERROR;
    return pBorder->get_Color(RHS);
}

HRESULT __stdcall CBorders::put_Color(VARIANT RHS)
{
	TRACINGMETHOD;
    for(int i = 0; i < m_indicesSize; ++i)
    {
        CComPtr<Border> pBorder(NULL);
        hr = get_Item(m_indices[i], &pBorder);
        CHECKERROR;
        hr = pBorder->put_Color(RHS);
        CHECKERROR;
    }
 	return hr;
}

HRESULT __stdcall CBorders::get_ColorIndex(VARIANT *RHS)
{
    TRACINGMETHOD;
    CComPtr<Border> pBorder(NULL);
    hr = get_Item(xlEdgeTop, &pBorder);
    CHECKERROR;
    return pBorder->get_ColorIndex(RHS);
}

HRESULT __stdcall CBorders::put_ColorIndex(VARIANT RHS)
{
    TRACINGMETHOD;
    for(int i = 0; i < m_indicesSize; ++i)
    {
        CComPtr<Border> pBorder(NULL);
        hr = get_Item(m_indices[i], &pBorder);
        CHECKERROR;
        hr = pBorder->put_ColorIndex(RHS);
        CHECKERROR;
    }
    return hr;
}

HRESULT __stdcall CBorders::get_Count(long *RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT __stdcall CBorders::get_Item(XlBordersIndex Index, Border **RHS)
{
	TRACINGMETHOD;

    CBorderObject* pBorderObject(NULL);
    hr = CBorderObject::CreateInstance(&pBorderObject);
    CHECKERROR;

    CBorderPtr pBorder(pBorderObject);

    hr = pBorder->Initialize(this, m_pOOParentRange, m_pTableBorder.p, Index);
    CHECKERROR;

    (*RHS) = pBorder;
    (*RHS)->AddRef();

    return hr;

}

HRESULT __stdcall CBorders::get_LineStyle(VARIANT *RHS)
{
    TRACINGMETHOD;
    CComPtr<Border> pBorder(NULL);
    hr = get_Item(xlEdgeTop, &pBorder);
    CHECKERROR;
    return pBorder->get_LineStyle(RHS);
}

HRESULT __stdcall CBorders::put_LineStyle(VARIANT RHS)
{
    TRACINGMETHOD;
    for(int i = 0; i < m_indicesSize; ++i)
    {
        CComPtr<Border> pBorder(NULL);
        hr = get_Item(m_indices[i], &pBorder);
        CHECKERROR;
        hr = pBorder->put_LineStyle(RHS);
        CHECKERROR;
    }
    return hr;
}

HRESULT __stdcall CBorders::get__NewEnum(IUnknown **RHS)
{
	TRACINGMETHOD;
    TRACINGNOTIMPL;
	return S_OK;
}

HRESULT __stdcall CBorders::get_Value(VARIANT *RHS)
{
	TRACINGMETHOD;
    return get_LineStyle(RHS);
}

HRESULT __stdcall CBorders::put_Value(VARIANT RHS)
{
	TRACINGMETHOD;
	return put_LineStyle(RHS);
}

HRESULT __stdcall CBorders::get_Weight(VARIANT *RHS)
{
    TRACINGMETHOD;
    CComPtr<Border> pBorder(NULL);
    hr = get_Item(xlEdgeTop, &pBorder);
    CHECKERROR;
    return pBorder->get_Weight(RHS);
}

HRESULT __stdcall CBorders::put_Weight(VARIANT RHS)
{
    TRACINGMETHOD;
    for(int i = 0; i < m_indicesSize; ++i)
    {
        CComPtr<Border> pBorder(NULL);
        hr = get_Item(m_indices[i], &pBorder);
        CHECKERROR;
        hr = pBorder->put_Weight(RHS);
        CHECKERROR;
    }
    return hr;
}

HRESULT __stdcall CBorders::get__Default(XlBordersIndex Index, Border **RHS)
{
	TRACINGMETHOD;
    hr = get_Item(Index, RHS);
    CHECKERROR;
	return hr;
}
