/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _RANGE_H_
#define _RANGE_H_

#include <atltypes.h>
#include <atlsafe.h>
#include "ExcelClone.h"
#include "DispatchHook.h"
#include "RangeAddress.h"
#include "Worksheet.h"
#include "OORange.h"
#include "Borders.h"

/*
	CRange class represents a cell, a row, a column, a selection of cells containing 
	one or more contiguous blocks of cells, or a 3-D range. It implements Range COM interface of the MS Excel.
*/

namespace AutomateIT
{
	namespace ExcelClone
	{

		class ATL_NO_VTABLE CRange :
			public CComObjectRootEx<CComSingleThreadModel>,
			public CDispatchHook< IDispatchImpl<Range, &IID_Range, &LIBID_ExcelCloneLib, 1, 0> >
		{
		public:
			CRange();
			virtual ~CRange();

			BEGIN_COM_MAP(CRange)
				COM_INTERFACE_ENTRY(Range)
				COM_INTERFACE_ENTRY(IDispatch)
			END_COM_MAP()

		public:



            HRESULT __stdcall get_Application( 
                 _Application **RHS);

            HRESULT __stdcall get_Creator( 
                 XlCreator *RHS);

            HRESULT __stdcall get_Parent( 
                 IDispatch **RHS);

            HRESULT __stdcall Activate( 
                 VARIANT *RHS);

            HRESULT __stdcall get_AddIndent( 
                 VARIANT *RHS);

            HRESULT __stdcall put_AddIndent( 
                 VARIANT RHS);

            HRESULT __stdcall get_Address( 
                 VARIANT RowAbsolute,
                 VARIANT ColumnAbsolute,
                 XlReferenceStyle ReferenceStyle,
                 VARIANT External,
                 VARIANT RelativeTo,
                 long lcid,
                 BSTR *RHS);

            HRESULT __stdcall get_AddressLocal( 
                 VARIANT RowAbsolute,
                 VARIANT ColumnAbsolute,
                 XlReferenceStyle ReferenceStyle,
                 VARIANT External,
                 VARIANT RelativeTo,
                 BSTR *RHS);

            HRESULT __stdcall AdvancedFilter( 
                 XlFilterAction Action,
                 VARIANT CriteriaRange,
                 VARIANT CopyToRange,
                 VARIANT Unique,
                 VARIANT *RHS);

            HRESULT __stdcall ApplyNames( 
                 VARIANT Names,
                 VARIANT IgnoreRelativeAbsolute,
                 VARIANT UseRowColumnNames,
                 VARIANT OmitColumn,
                 VARIANT OmitRow,
                 XlApplyNamesOrder Order,
                 VARIANT AppendLast,
                 VARIANT *RHS);

            HRESULT __stdcall ApplyOutlineStyles( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Areas( 
                 IDispatch **RHS);

            HRESULT __stdcall AutoComplete( 
                 BSTR String,
                 BSTR *RHS);

            HRESULT __stdcall AutoFill( 
                 Range *Destination,
                 XlAutoFillType Type,
                 VARIANT *RHS);

            HRESULT __stdcall AutoFilter( 
                 VARIANT Field,
                 VARIANT Criteria1,
                 XlAutoFilterOperator Operator,
                 VARIANT Criteria2,
                 VARIANT VisibleDropDown,
                 VARIANT *RHS);

            HRESULT __stdcall AutoFit( 
                 VARIANT *RHS);

            HRESULT __stdcall AutoFormat( 
                 XlRangeAutoFormat Format,
                 VARIANT Number,
                 VARIANT IFont,
                 VARIANT Alignment,
                 VARIANT Border,
                 VARIANT Pattern,
                 VARIANT Width,
                 VARIANT *RHS);

            HRESULT __stdcall AutoOutline( 
                 VARIANT *RHS);

            HRESULT __stdcall BorderAround( 
                 VARIANT LineStyle,
                 XlBorderWeight Weight,
                 XlColorIndex ColorIndex,
                 VARIANT Color,
                 VARIANT *RHS);

            HRESULT __stdcall get_Borders( 
                 Borders **RHS);

            HRESULT __stdcall Calculate( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Cells( 
                 Range **RHS);

            HRESULT __stdcall get_Characters( 
                 VARIANT Start,
                 VARIANT Length,
                 IDispatch **RHS);

            HRESULT __stdcall CheckSpelling( 
                 VARIANT CustomDictionary,
                 VARIANT IgnoreUppercase,
                 VARIANT AlwaysSuggest,
                 VARIANT SpellLang,
                 VARIANT *RHS);

            HRESULT __stdcall Clear( 
                 VARIANT *RHS);

            HRESULT __stdcall ClearContents( 
                 VARIANT *RHS);

            HRESULT __stdcall ClearFormats( 
                 VARIANT *RHS);

            HRESULT __stdcall ClearNotes( 
                 VARIANT *RHS);

            HRESULT __stdcall ClearOutline( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Column( 
                 long *RHS);

            HRESULT __stdcall ColumnDifferences( 
                 VARIANT Comparison,
                 Range **RHS);

            HRESULT __stdcall get_Columns( 
                 Range **RHS);

            HRESULT __stdcall get_ColumnWidth( 
                 VARIANT *RHS);

            HRESULT __stdcall put_ColumnWidth( 
                 VARIANT RHS);

            HRESULT __stdcall Consolidate( 
                 VARIANT Sources,
                 VARIANT Function,
                 VARIANT TopRow,
                 VARIANT LeftColumn,
                 VARIANT CreateLinks,
                 VARIANT *RHS);

            HRESULT __stdcall Copy( 
                 VARIANT Destination,
                 VARIANT *RHS);

            HRESULT __stdcall CopyFromRecordset( 
                 IUnknown *Data,
                 VARIANT MaxRows,
                 VARIANT MaxColumns,
                 long *RHS);

            HRESULT __stdcall CopyPicture( 
                 XlPictureAppearance Appearance,
                 XlCopyPictureFormat Format,
                 VARIANT *RHS);

            HRESULT __stdcall get_Count( 
                 long *RHS);

            HRESULT __stdcall CreateNames( 
                 VARIANT Top,
                 VARIANT Left,
                 VARIANT Bottom,
                 VARIANT Right,
                 VARIANT *RHS);

            HRESULT __stdcall CreatePublisher( 
                 VARIANT Edition,
                 XlPictureAppearance Appearance,
                 VARIANT ContainsPICT,
                 VARIANT ContainsBIFF,
                 VARIANT ContainsRTF,
                 VARIANT ContainsVALU,
                 VARIANT *RHS);

            HRESULT __stdcall get_CurrentArray( 
                 Range **RHS);

            HRESULT __stdcall get_CurrentRegion( 
                 Range **RHS);

            HRESULT __stdcall Cut( 
                 VARIANT Destination,
                 VARIANT *RHS);

            HRESULT __stdcall DataSeries( 
                 VARIANT Rowcol,
                 XlDataSeriesType Type,
                 XlDataSeriesDate Date,
                 VARIANT Step,
                 VARIANT Stop,
                 VARIANT Trend,
                 VARIANT *RHS);

            HRESULT __stdcall get__Default( 
                 VARIANT RowIndex,
                 VARIANT ColumnIndex,
                 long lcid,
                 VARIANT *RHS);

            HRESULT __stdcall put__Default( 
                 VARIANT RowIndex,
                 VARIANT ColumnIndex,
                 long lcid,
                 VARIANT RHS);

            HRESULT __stdcall Delete( 
                 VARIANT Shift,
                 VARIANT *RHS);

            HRESULT __stdcall get_Dependents( 
                 Range **RHS);

            HRESULT __stdcall DialogBox1( 
                 VARIANT *RHS);

            HRESULT __stdcall get_DirectDependents( 
                 Range **RHS);

            HRESULT __stdcall get_DirectPrecedents( 
                 Range **RHS);

            HRESULT __stdcall EditionOptions( 
                 XlEditionType Type,
                 XlEditionOptionsOption Option,
                 VARIANT Name,
                 VARIANT Reference,
                 XlPictureAppearance Appearance,
                 XlPictureAppearance ChartSize,
                 VARIANT Format,
                 VARIANT *RHS);

            HRESULT __stdcall get_End( 
                 XlDirection Direction,
                 Range **RHS);

            HRESULT __stdcall get_EntireColumn( 
                 Range **RHS);

            HRESULT __stdcall get_EntireRow( 
                 Range **RHS);

            HRESULT __stdcall FillDown( 
                 VARIANT *RHS);

            HRESULT __stdcall FillLeft( 
                 VARIANT *RHS);

            HRESULT __stdcall FillRight( 
                 VARIANT *RHS);

            HRESULT __stdcall FillUp( 
                 VARIANT *RHS);

            HRESULT __stdcall Find( 
                 VARIANT What,
                 VARIANT After,
                 VARIANT LookIn,
                 VARIANT LookAt,
                 VARIANT SearchOrder,
                 XlSearchDirection SearchDirection,
                 VARIANT MatchCase,
                 VARIANT MatchByte,
                 VARIANT SearchFormat,
                 Range **RHS);

            HRESULT __stdcall FindNext( 
                 VARIANT After,
                 Range **RHS);

            HRESULT __stdcall FindPrevious( 
                 VARIANT After,
                 Range **RHS);

            HRESULT __stdcall get_Font( 
                 Font **RHS);

            HRESULT __stdcall get_Formula( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT __stdcall put_Formula( 
                 long lcid,
                 VARIANT RHS);

            HRESULT __stdcall get_FormulaArray( 
                 VARIANT *RHS);

            HRESULT __stdcall put_FormulaArray( 
                 VARIANT RHS);

            HRESULT __stdcall get_FormulaLabel( 
                 XlFormulaLabel *RHS);

            HRESULT __stdcall put_FormulaLabel( 
                 XlFormulaLabel RHS);

            HRESULT __stdcall get_FormulaHidden( 
                 VARIANT *RHS);

            HRESULT __stdcall put_FormulaHidden( 
                 VARIANT RHS);

            HRESULT __stdcall get_FormulaLocal( 
                 VARIANT *RHS);

            HRESULT __stdcall put_FormulaLocal( 
                 VARIANT RHS);

            HRESULT __stdcall get_FormulaR1C1( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT __stdcall put_FormulaR1C1( 
                 long lcid,
                 VARIANT RHS);

            HRESULT __stdcall get_FormulaR1C1Local( 
                 VARIANT *RHS);

            HRESULT __stdcall put_FormulaR1C1Local( 
                 VARIANT RHS);

            HRESULT __stdcall FunctionWizard( 
                 VARIANT *RHS);

            HRESULT __stdcall GoalSeek( 
                 VARIANT Goal,
                 Range *ChangingCell,
                 VARIANT_BOOL *RHS);

            HRESULT __stdcall Group( 
                 VARIANT Start,
                 VARIANT End,
                 VARIANT By,
                 VARIANT Periods,
                 VARIANT *RHS);

            HRESULT __stdcall get_HasArray( 
                 VARIANT *RHS);

            HRESULT __stdcall get_HasFormula( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Height( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Hidden( 
                 VARIANT *RHS);

            HRESULT __stdcall put_Hidden( 
                 VARIANT RHS);

            HRESULT __stdcall get_HorizontalAlignment( 
                 VARIANT *RHS);

            HRESULT __stdcall put_HorizontalAlignment( 
                 VARIANT RHS);

            HRESULT __stdcall get_IndentLevel( 
                 VARIANT *RHS);

            HRESULT __stdcall put_IndentLevel( 
                 VARIANT RHS);

            HRESULT __stdcall InsertIndent( 
                 long InsertAmount);

            HRESULT __stdcall Insert( 
                 VARIANT Shift,
                 VARIANT CopyOrigin,
                 VARIANT *RHS);

            HRESULT __stdcall get_Interior( 
                 Interior **RHS);

            HRESULT __stdcall get_Item( 
                 VARIANT RowIndex,
                 VARIANT ColumnIndex,
                 long lcid,
                 VARIANT *RHS);

            HRESULT __stdcall put_Item( 
                 VARIANT RowIndex,
                 VARIANT ColumnIndex,
                 long lcid,
                 VARIANT RHS);

            HRESULT __stdcall Justify( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Left( 
                 VARIANT *RHS);

            HRESULT __stdcall get_ListHeaderRows( 
                 long *RHS);

            HRESULT __stdcall ListNames( 
                 VARIANT *RHS);

            HRESULT __stdcall get_LocationInTable( 
                 XlLocationInTable *RHS);

            HRESULT __stdcall get_Locked( 
                 VARIANT *RHS);

            HRESULT __stdcall put_Locked( 
                 VARIANT RHS);

            HRESULT __stdcall Merge( 
                 VARIANT Across);

            HRESULT __stdcall UnMerge( void);

            HRESULT __stdcall get_MergeArea( 
                 Range **RHS);

            HRESULT __stdcall get_MergeCells( 
                 VARIANT *RHS);

            HRESULT __stdcall put_MergeCells( 
                 VARIANT RHS);

            HRESULT __stdcall get_Name( 
                 VARIANT *RHS);

            HRESULT __stdcall put_Name( 
                 VARIANT RHS);

            HRESULT __stdcall NavigateArrow( 
                 VARIANT TowardPrecedent,
                 VARIANT ArrowNumber,
                 VARIANT LinkNumber,
                 VARIANT *RHS);

            HRESULT __stdcall get__NewEnum( 
                 IUnknown **RHS);

            HRESULT __stdcall get_Next( 
                 Range **RHS);

            HRESULT __stdcall NoteText( 
                 VARIANT Text,
                 VARIANT Start,
                 VARIANT Length,
                 BSTR *RHS);

            HRESULT __stdcall get_NumberFormat( 
                 VARIANT *RHS);

            HRESULT __stdcall put_NumberFormat( 
                 VARIANT RHS);

            HRESULT __stdcall get_NumberFormatLocal( 
                 VARIANT *RHS);

            HRESULT __stdcall put_NumberFormatLocal( 
                 VARIANT RHS);

            HRESULT __stdcall get_Offset( 
                 VARIANT RowOffset,
                 VARIANT ColumnOffset,
                 Range **RHS);

            HRESULT __stdcall get_Orientation( 
                 VARIANT *RHS);

            HRESULT __stdcall put_Orientation( 
                 VARIANT RHS);

            HRESULT __stdcall get_OutlineLevel( 
                 VARIANT *RHS);

            HRESULT __stdcall put_OutlineLevel( 
                 VARIANT RHS);

            HRESULT __stdcall get_PageBreak( 
                 long *RHS);

            HRESULT __stdcall put_PageBreak( 
                 long RHS);

            HRESULT __stdcall Parse( 
                 VARIANT ParseLine,
                 VARIANT Destination,
                 VARIANT *RHS);

            HRESULT __stdcall _PasteSpecial( 
                 XlPasteType Paste,
                 XlPasteSpecialOperation Operation,
                 VARIANT SkipBlanks,
                 VARIANT Transpose,
                 VARIANT *RHS);

            HRESULT __stdcall get_PivotField( 
                 IDispatch **RHS);

            HRESULT __stdcall get_PivotItem( 
                 IDispatch **RHS);

            HRESULT __stdcall get_PivotTable( 
                 IDispatch **RHS);

            HRESULT __stdcall get_Precedents( 
                 Range **RHS);

            HRESULT __stdcall get_PrefixCharacter( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Previous( 
                 Range **RHS);

            HRESULT __stdcall _PrintOut( 
                 VARIANT From,
                 VARIANT To,
                 VARIANT Copies,
                 VARIANT Preview,
                 VARIANT ActivePrinter,
                 VARIANT PrintToFile,
                 VARIANT Collate,
                 VARIANT *RHS);

            HRESULT __stdcall PrintPreview( 
                 VARIANT EnableChanges,
                 VARIANT *RHS);

            HRESULT __stdcall get_QueryTable( 
                 IDispatch **RHS);

            HRESULT __stdcall get_Range( 
                 VARIANT Cell1,
                 VARIANT Cell2,
                 Range **RHS);

            HRESULT __stdcall RemoveSubtotal( 
                 VARIANT *RHS);

            HRESULT __stdcall Replace( 
                 VARIANT What,
                 VARIANT Replacement,
                 VARIANT LookAt,
                 VARIANT SearchOrder,
                 VARIANT MatchCase,
                 VARIANT MatchByte,
                 VARIANT SearchFormat,
                 VARIANT ReplaceFormat,
                 VARIANT_BOOL *RHS);

            HRESULT __stdcall get_Resize( 
                 VARIANT RowSize,
                 VARIANT ColumnSize,
                 Range **RHS);

            HRESULT __stdcall get_Row( 
                 long *RHS);

            HRESULT __stdcall RowDifferences( 
                 VARIANT Comparison,
                 Range **RHS);

            HRESULT __stdcall get_RowHeight( 
                 VARIANT *RHS);

            HRESULT __stdcall put_RowHeight( 
                 VARIANT RHS);

            HRESULT __stdcall get_Rows( 
                 Range **RHS);

            HRESULT __stdcall Run( 
                 VARIANT Arg1,
                 VARIANT Arg2,
                 VARIANT Arg3,
                 VARIANT Arg4,
                 VARIANT Arg5,
                 VARIANT Arg6,
                 VARIANT Arg7,
                 VARIANT Arg8,
                 VARIANT Arg9,
                 VARIANT Arg10,
                 VARIANT Arg11,
                 VARIANT Arg12,
                 VARIANT Arg13,
                 VARIANT Arg14,
                 VARIANT Arg15,
                 VARIANT Arg16,
                 VARIANT Arg17,
                 VARIANT Arg18,
                 VARIANT Arg19,
                 VARIANT Arg20,
                 VARIANT Arg21,
                 VARIANT Arg22,
                 VARIANT Arg23,
                 VARIANT Arg24,
                 VARIANT Arg25,
                 VARIANT Arg26,
                 VARIANT Arg27,
                 VARIANT Arg28,
                 VARIANT Arg29,
                 VARIANT Arg30,
                 VARIANT *RHS);

            HRESULT __stdcall Select( 
                 VARIANT *RHS);

            HRESULT __stdcall Show( 
                 VARIANT *RHS);

            HRESULT __stdcall ShowDependents( 
                 VARIANT Remove,
                 VARIANT *RHS);

            HRESULT __stdcall get_ShowDetail( 
                 VARIANT *RHS);

            HRESULT __stdcall put_ShowDetail( 
                 VARIANT RHS);

            HRESULT __stdcall ShowErrors( 
                 VARIANT *RHS);

            HRESULT __stdcall ShowPrecedents( 
                 VARIANT Remove,
                 VARIANT *RHS);

            HRESULT __stdcall get_ShrinkToFit( 
                 VARIANT *RHS);

            HRESULT __stdcall put_ShrinkToFit( 
                 VARIANT RHS);

            HRESULT __stdcall Sort( 
                 VARIANT Key1,
                 XlSortOrder Order1,
                 VARIANT Key2,
                 VARIANT Type,
                 XlSortOrder Order2,
                 VARIANT Key3,
                 XlSortOrder Order3,
                 XlYesNoGuess Header,
                 VARIANT OrderCustom,
                 VARIANT MatchCase,
                 XlSortOrientation Orientation,
                 XlSortMethod SortMethod,
                 XlSortDataOption DataOption1,
                 XlSortDataOption DataOption2,
                 XlSortDataOption DataOption3,
                 VARIANT *RHS);

            HRESULT __stdcall SortSpecial( 
                 XlSortMethod SortMethod,
                 VARIANT Key1,
                 XlSortOrder Order1,
                 VARIANT Type,
                 VARIANT Key2,
                 XlSortOrder Order2,
                 VARIANT Key3,
                 XlSortOrder Order3,
                 XlYesNoGuess Header,
                 VARIANT OrderCustom,
                 VARIANT MatchCase,
                 XlSortOrientation Orientation,
                 XlSortDataOption DataOption1,
                 XlSortDataOption DataOption2,
                 XlSortDataOption DataOption3,
                 VARIANT *RHS);

            HRESULT __stdcall get_SoundNote( 
                 IDispatch **RHS);

            HRESULT __stdcall SpecialCells( 
                 XlCellType Type,
                 VARIANT Value,
                 Range **RHS);

            HRESULT __stdcall get_Style( 
                 VARIANT *RHS);

            HRESULT __stdcall put_Style( 
                 VARIANT RHS);

            HRESULT __stdcall SubscribeTo( 
                 BSTR Edition,
                 XlSubscribeToFormat Format,
                 VARIANT *RHS);

            HRESULT __stdcall Subtotal( 
                 long GroupBy,
                 XlConsolidationFunction Function,
                 VARIANT TotalList,
                 VARIANT Replace,
                 VARIANT PageBreaks,
                 XlSummaryRow SummaryBelowData,
                 VARIANT *RHS);

            HRESULT __stdcall get_Summary( 
                 VARIANT *RHS);

            HRESULT __stdcall Table( 
                 VARIANT RowInput,
                 VARIANT ColumnInput,
                 VARIANT *RHS);

            HRESULT __stdcall get_Text( 
                 VARIANT *RHS);

            HRESULT __stdcall TextToColumns( 
                 VARIANT Destination,
                 XlTextParsingType DataType,
                 XlTextQualifier TextQualifier,
                 VARIANT ConsecutiveDelimiter,
                 VARIANT Tab,
                 VARIANT Semicolon,
                 VARIANT Comma,
                 VARIANT Space,
                 VARIANT Other,
                 VARIANT OtherChar,
                 VARIANT FieldInfo,
                 VARIANT DecimalSeparator,
                 VARIANT ThousandsSeparator,
                 VARIANT TrailingMinusNumbers,
                 VARIANT *RHS);

            HRESULT __stdcall get_Top( 
                 VARIANT *RHS);

            HRESULT __stdcall Ungroup( 
                 VARIANT *RHS);

            HRESULT __stdcall get_UseStandardHeight( 
                 VARIANT *RHS);

            HRESULT __stdcall put_UseStandardHeight( 
                 VARIANT RHS);

            HRESULT __stdcall get_UseStandardWidth( 
                 VARIANT *RHS);

            HRESULT __stdcall put_UseStandardWidth( 
                 VARIANT RHS);

            HRESULT __stdcall get_Validation( 
                 IDispatch **RHS);

            HRESULT __stdcall get_Value( 
                 VARIANT RangeValueDataType,
                 long lcid,
                 VARIANT *RHS);

            HRESULT __stdcall put_Value( 
                 VARIANT RangeValueDataType,
                 long lcid,
                 VARIANT RHS);

            HRESULT __stdcall get_Value2( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT __stdcall put_Value2( 
                 long lcid,
                 VARIANT RHS);

            HRESULT __stdcall get_VerticalAlignment( 
                 VARIANT *RHS);

            HRESULT __stdcall put_VerticalAlignment( 
                 VARIANT RHS);

            HRESULT __stdcall get_Width( 
                 VARIANT *RHS);

            HRESULT __stdcall get_Worksheet( 
                 _Worksheet **RHS);

            HRESULT __stdcall get_WrapText( 
                 VARIANT *RHS);

            HRESULT __stdcall put_WrapText( 
                 VARIANT RHS);

            HRESULT __stdcall AddComment( 
                 VARIANT Text,
                 IDispatch **RHS);

            HRESULT __stdcall get_Comment( 
                 IDispatch **RHS);

            HRESULT __stdcall ClearComments( void);

            HRESULT __stdcall get_Phonetic( 
                 IDispatch **RHS);

            HRESULT __stdcall get_FormatConditions( 
                 IDispatch **RHS);

            HRESULT __stdcall get_ReadingOrder( 
                 long *RHS);

            HRESULT __stdcall put_ReadingOrder( 
                 long RHS);

            HRESULT __stdcall get_Hyperlinks( 
                 IDispatch **RHS);

            HRESULT __stdcall get_Phonetics( 
                 IDispatch **RHS);

            HRESULT __stdcall SetPhonetic( void);

            HRESULT __stdcall get_ID( 
                 BSTR *RHS);

            HRESULT __stdcall put_ID( 
                 BSTR RHS);

            HRESULT __stdcall PrintOut( 
                 VARIANT From,
                 VARIANT To,
                 VARIANT Copies,
                 VARIANT Preview,
                 VARIANT ActivePrinter,
                 VARIANT PrintToFile,
                 VARIANT Collate,
                 VARIANT PrToFileName,
                 VARIANT *RHS);

            HRESULT __stdcall get_PivotCell( 
                 IDispatch **RHS);

            HRESULT __stdcall Dirty( void);

            HRESULT __stdcall get_Errors( 
                 IDispatch **RHS);

            HRESULT __stdcall get_SmartTags( 
                 IDispatch **RHS);

            HRESULT __stdcall Speak( 
                 VARIANT SpeakDirection,
                 VARIANT SpeakFormulas);

            HRESULT __stdcall PasteSpecial( 
                 XlPasteType Paste,
                 XlPasteSpecialOperation Operation,
                 VARIANT SkipBlanks,
                 VARIANT Transpose,
                 VARIANT *RHS);

            HRESULT __stdcall get_AllowEdit( 
                 VARIANT_BOOL *RHS);

            HRESULT __stdcall get_ListObject( 
                 IDispatch **RHS);

            HRESULT __stdcall get_XPath( 
                 IDispatch **RHS);

        private:

            template <typename T>
            HRESULT __stdcall SetArrayValue(CComVariant vValue)
            {
                TRACINGMETHOD;

                CComSafeArray<T> arrayValues(*(vValue.parray));

                CRangeAddress address(OLESTR("A1:A1"));
                hr = GetRangeAddress(address);
                CHECKERROR;

                if (arrayValues.GetDimensions() == 1) 
                {
                    // column
                    if (address.GetLeft() == address.GetRight())
                    {
                        for (long i = 0; i < static_cast<long>(arrayValues.GetCount(0)); ++i)
                        {
                            if ( i <= (address.GetBottom() - address.GetTop())) 
                            {
                                CComVariant result;
                                hr = get__Default(CComVariant(i + 1), CComVariant(1), 0, &result);
                                CHECKERROR;
                                CRangePtr pRange((CRange*)result.pdispVal);

                                hr = pRange->put_Value(CComVariant(), 0, CComVariant((T)arrayValues.GetAt(i)));
                                CHECKERROR;
                            }
                        }
                    }
                    // row
                    if (address.GetTop() == address.GetBottom())
                    {
                        for (long i = 0; i < static_cast<long>(arrayValues.GetCount(0)); ++i)
                        {
                            if ( i <= (address.GetRight() - address.GetLeft())) 
                            {
                                CComVariant result;
                                hr = get__Default(CComVariant(1), CComVariant(i + 1),  0, &result);
                                CHECKERROR;
                                CRangePtr pRange((CRange*)result.pdispVal);

                                hr = pRange->put_Value(CComVariant(), 0, CComVariant((T)arrayValues.GetAt(i)));
                                CHECKERROR;
                            }
                        }
                    }
                    return hr;
                }
                if (arrayValues.GetDimensions() == 2)
                {
                    for (long i = 0; i < static_cast<long>(arrayValues.GetCount(0)); ++i)
                    {
                        for (long j = 0; j < static_cast<long>(arrayValues.GetCount(1)); ++j)
                        {
                            if (( i <= (address.GetBottom() - address.GetTop())) && 
                                (j <= (address.GetRight() - address.GetLeft()))) 
                            {
                                CComVariant result;
                                hr = get__Default(CComVariant(i + 1), CComVariant(j + 1), 0, &result);
                                CHECKERROR;
                                CRangePtr pRange((CRange*)result.pdispVal);

                                hr = pRange->put_Value(CComVariant(), 0, CComVariant((T)arrayValues.GetAt(j + i * arrayValues.GetCount(0))));
                                CHECKERROR;
                            }
                        }
                    }
                }
                return hr;
            }


		public:

            STDMETHOD(GetRangeAddress)(CalcItems::CRangeAddress& address);
            STDMETHOD(Initialize)(CApplication* application, CWorksheet* sheet, OpenOfficeWrapper::COORangePtr pOORange, const std::wstring& name);
			STDMETHOD(Initialize)(CApplication* application, CWorksheet* sheet, OpenOfficeWrapper::COORangePtr pOORange, const CPoint& topLeft, const CPoint& bottomRight);
			STDMETHOD(Initialize)(CApplication* application, CWorksheet* sheet, OpenOfficeWrapper::COORangePtr pOORange);

            OpenOfficeWrapper::COORangePtr GetRangeObject() const;

            void SetInternalType(XlRowCol internalType);

		private:
			
            OpenOfficeWrapper::COORangePtr m_pRange;
            OpenOfficeWrapper::DispatchList m_garbageList;
            CWorksheet* m_pSheet;
    		CApplication* m_ParentApplication;
            CBordersPtr m_pBorders;

            XlRowCol m_InternalType;
        };

		typedef CComObject<CRange> CRangeObject;
		typedef CComPtr<CRange> CRangePtr;
	}
}

#endif //_RANGE_H_
