
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "Sheets.h"

using namespace AutomateIT;
using namespace AutomateIT::Utils;
using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::OpenOfficeWrapper;


CSheets::CSheets() :
	m_ParentApplication(NULL),
	m_ParentWorkbook(NULL)
{
    TRACINGMETHOD; 
}

CSheets::~CSheets()
{
    TRACINGMETHOD; 
}

STDMETHODIMP CSheets::Initialize(CApplication* pParentApplication, CWorkbook* pWorkbook)
{
    TRACINGMETHOD;
    m_ParentApplication = pParentApplication;
    m_ParentWorkbook = pWorkbook;
    hr = m_ParentWorkbook->GetOODocument()->GetSheets(&m_pSheets);
    CHECKERROR;
    return hr;
}


HRESULT STDMETHODCALLTYPE CSheets::GetActiveSheet(CWorksheet** ppWorksheet)
{
    TRACINGMETHOD;
    CComVariant resultSheet; 

    COORangePtr pOOSheetPtr(NULL);
    hr = m_ParentWorkbook->GetOODocument()->get_ActiveSheet(&pOOSheetPtr);
	CHECKERROR;
    CWorksheetObject* pWorkSheet(NULL);
    hr = CWorksheetObject::CreateInstance(&pWorkSheet);
    CHECKERROR;
    CWorksheetPtr pWorkSheetPtr(pWorkSheet);
    pWorkSheetPtr->Initialize(m_ParentApplication, m_ParentWorkbook, pOOSheetPtr);
    *ppWorksheet = pWorkSheetPtr;
    (*ppWorksheet)->AddRef();
    return hr;
}

OpenOfficeWrapper::COOSheetsPtr CSheets::GetOOSheetObject()
{
    return m_pSheets;
}



HRESULT STDMETHODCALLTYPE CSheets::get_Application( 
    _Application **RHS)
{
    TRACINGMETHOD; 
    (*RHS) = m_ParentApplication;
    (*RHS)->AddRef();
    return hr; 
}

HRESULT STDMETHODCALLTYPE CSheets::get_Creator( 
                                      XlCreator *RHS)
{ 
    TRACINGMETHOD;
    (*RHS) = xlCreatorCode;
    return hr;
}

HRESULT STDMETHODCALLTYPE CSheets::get_Parent( 
                                     IDispatch **RHS)
{ 
    TRACINGMETHOD;
    (*RHS) = m_ParentWorkbook;
    (*RHS)->AddRef();
    return hr;
}

HRESULT STDMETHODCALLTYPE CSheets::Add( 
                              VARIANT Before,
                              VARIANT After,
                              VARIANT Count,
                              VARIANT Type,
                              long lcid,
                              IDispatch **ppNewSheet)
{ 
    long where(0);
    TRACINGMETHOD; 
    if (Before.vt == VT_BSTR)
    {
        COORangePtr pSheet(NULL);
        hr = m_pSheets->GetByName(Before.bstrVal, &pSheet);
        CHECKERROR;
        COORangeAddressPtr pRangeAddress(NULL);
        hr = pSheet->GetRangeAddress(&pRangeAddress);
        CHECKERROR;
        hr = pRangeAddress->get_Sheet(&where);
        CHECKERROR;
    }
    else if (After.vt == VT_BSTR)
    {
        COORangePtr pSheet(NULL);
        hr = m_pSheets->GetByName(After.bstrVal, &pSheet);
        CHECKERROR;
        COORangeAddressPtr pRangeAddress(NULL);
        hr = pSheet->GetRangeAddress(&pRangeAddress);
        CHECKERROR;
        hr = pRangeAddress->get_Sheet(&where);
        CHECKERROR;
        where++;
   }
    else
    {
        where = 0;
    }

    hr = m_pSheets->InsertNewByName(OLESTR("new sheet"), where);
    CHECKERROR;

    hr = get__Default(CComVariant(OLESTR("new sheet")), ppNewSheet);
    CHECKERROR;

    return hr;
}

HRESULT STDMETHODCALLTYPE CSheets::Copy( 
                               VARIANT Before,
                               VARIANT After,
                               long lcid)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::get_Count( 
                                    long *RHS)
{ 
    TRACINGMETHOD; 
    hr = m_pSheets->Count(RHS);
    CHECKERROR;
    return hr; 
}

HRESULT STDMETHODCALLTYPE CSheets::Delete( 
                                 long lcid)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::FillAcrossSheets( 
    Range *Range,
    XlFillWith Type,
    long lcid)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::get_Item( 
                                   VARIANT Index,
                                   IDispatch **RHS)
{ 
    TRACINGMETHOD; 
    return get__Default(Index, RHS); 
}

HRESULT STDMETHODCALLTYPE CSheets::Move( 
                               VARIANT Before,
                               VARIANT After,
                               long lcid)
{ 
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::get__NewEnum( 
                                       IUnknown **RHS)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::_PrintOut( 
                                    VARIANT From,
                                    VARIANT To,
                                    VARIANT Copies,
                                    VARIANT Preview,
                                    VARIANT ActivePrinter,
                                    VARIANT PrintToFile,
                                    VARIANT Collate,
                                    long lcid)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::PrintPreview( 
                                       VARIANT EnableChanges,
                                       long lcid)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::Select( 
                                 VARIANT Replace,
                                 long lcid)
{
    TRACINGMETHOD;
    IDispatchPtr pdSheet(NULL);
    hr = get_Item(CComVariant(1), &pdSheet);
    CHECKERROR;

    CComPtr<_Worksheet> pSheet(NULL);

    hr = pdSheet.QueryInterface(&pSheet);
    CHECKERROR;

    hr = pSheet->Select(Replace, lcid);
    CHECKERROR;

    return hr; 
}

HRESULT STDMETHODCALLTYPE CSheets::get_HPageBreaks( 
    IDispatch **RHS)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::get_VPageBreaks( 
    IDispatch **RHS)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::get_Visible( 
                                      long lcid,
                                      VARIANT *RHS)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::put_Visible( 
                                      long lcid,
                                      VARIANT RHS)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CSheets::get__Default( 
                                       VARIANT Index,
                                       IDispatch **ppSheet)
{ 
    TRACINGMETHOD; 

    COORangePtr range(NULL);


    CComVariant index(Index);
    hr = index.ChangeType(VT_I4);
    if (SUCCEEDED(hr))
    {
        index.lVal--;
        hr = m_pSheets->GetByIndex(index.lVal, &range);
        CHECKERROR;
    }
    else
    {
        hr = index.ChangeType(VT_BSTR);
        if (SUCCEEDED(hr))
        {
            std::wstring name = index.bstrVal;
            std::replace(name.begin(), name.end(), L',', L'_');

            hr = m_pSheets->GetByName(CComBSTR(name.c_str()), &range);
            CHECKERROR;
        }
        else
        {
            *ppSheet = NULL;
            hr = E_OUTOFRANGE;
            return hr;
        }
    }

    CWorksheetObject* pSheet(NULL);
    hr = CWorksheetObject::CreateInstance(&pSheet);
    CHECKERROR;

    CWorksheetPtr pSheetPtr(pSheet);
    hr = pSheetPtr->Initialize(m_ParentApplication, m_ParentWorkbook, range);
    CHECKERROR;

    m_garbageList.AddTail(pSheetPtr.p);

    (*ppSheet) = pSheetPtr;
    (*ppSheet)->AddRef();

    return hr;
}

HRESULT STDMETHODCALLTYPE CSheets::PrintOut( 
                                   VARIANT From,
                                   VARIANT To,
                                   VARIANT Copies,
                                   VARIANT Preview,
                                   VARIANT ActivePrinter,
                                   VARIANT PrintToFile,
                                   VARIANT Collate,
                                   VARIANT PrToFileName,
                                   long lcid)
{
    TRACINGMETHOD; 
    TRACINGNOTIMPL;
    return S_OK; 
}

