
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "StdAfx.h"
#include "Workbook.h"

using namespace AutomateIT;
using namespace AutomateIT::Utils;
using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::OpenOfficeWrapper;

CWorkbook::CWorkbook()
{
    TRACINGMETHOD;
}

CWorkbook::~CWorkbook()
{
    TRACINGMETHOD;
}

STDMETHODIMP CWorkbook::Initialize(CApplication* pApp)
{
    TRACINGMETHOD;
    m_ParentApplication = pApp;

    COODocumentPtr pOOBaseDocument(NULL);
    m_ParentApplication->GetOOApplication()->CreateBlankDocument(Strings::TypeCalcDocument, &pOOBaseDocument, TRUE);

    COOCalcDocumentObject* pOOCalcDocument(NULL);
    hr = COOCalcDocumentObject::CreateInstance(&pOOCalcDocument);
    CHECKERROR;

    COOCalcDocumentPtr pOOCalcDocumentPtr(pOOCalcDocument);
    hr = pOOCalcDocumentPtr->Attach(pOOBaseDocument);
    CHECKERROR;

    CSheetsObject* pSheets(NULL);
    hr = CSheetsObject::CreateInstance(&pSheets);
    CHECKERROR;

    CSheetsPtr pSheetsPtr(pSheets);

    m_pOOCalcDocument = pOOCalcDocumentPtr;
    m_Sheets = pSheetsPtr;

    hr = pSheetsPtr->Initialize(m_ParentApplication, this);
    CHECKERROR;


    return hr;
}

STDMETHODIMP CWorkbook::Initialize(CApplication* pApp, IDispatchPtr pDocument)
{
    TRACINGMETHOD;
    m_ParentApplication = pApp;

    COODocumentObject* pOOBaseDocumentObject(NULL);
    hr = COODocumentObject::CreateInstance(&pOOBaseDocumentObject);
    CHECKERROR;

    COODocumentPtr pOOBaseDocument(pOOBaseDocumentObject);

    hr = pOOBaseDocument->Initialize(m_ParentApplication->GetOOApplication().p, pDocument);
    CHECKERROR;

    COOCalcDocumentObject* pOOCalcDocument(NULL);
    hr = COOCalcDocumentObject::CreateInstance(&pOOCalcDocument);
    CHECKERROR;

    COOCalcDocumentPtr pOOCalcDocumentPtr(pOOCalcDocument);
    hr = pOOCalcDocumentPtr->Attach(pOOBaseDocument);
    CHECKERROR;

    CSheetsObject* pSheets(NULL);
    hr = CSheetsObject::CreateInstance(&pSheets);
    CHECKERROR;

    CSheetsPtr pSheetsPtr(pSheets);

    m_pOOCalcDocument = pOOCalcDocumentPtr;
    m_Sheets = pSheetsPtr;

    hr = pSheetsPtr->Initialize(m_ParentApplication, this);
    CHECKERROR;


    return hr;
}


STDMETHODIMP CWorkbook::Initialize(CApplication* pApp, CComVariant vFileName)
{
    TRACINGMETHOD;

    m_ParentApplication = pApp;

    COODocumentPtr pOOBaseDocument(NULL);
    m_ParentApplication->GetOOApplication()->CreateBlankDocument(vFileName.bstrVal, &pOOBaseDocument, TRUE);

    COOCalcDocumentObject* pOOCalcDocument(NULL);
    hr = COOCalcDocumentObject::CreateInstance(&pOOCalcDocument);
    CHECKERROR;

    COOCalcDocumentPtr pOOCalcDocumentPtr(pOOCalcDocument);
    hr = pOOCalcDocumentPtr->Attach(pOOBaseDocument);
    CHECKERROR;

    CSheetsObject* pSheets(NULL);
    hr = CSheetsObject::CreateInstance(&pSheets);
    CHECKERROR;

    CSheetsPtr pSheetsPtr(pSheets);

    m_pOOCalcDocument = pOOCalcDocumentPtr;
    m_Sheets = pSheetsPtr;

    hr = pSheetsPtr->Initialize(m_ParentApplication, this);
    CHECKERROR;

    return hr;
}


OpenOfficeWrapper::COOCalcDocumentPtr CWorkbook::GetOODocument() const
{
    return m_pOOCalcDocument;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Application( 
    _Application **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Creator( 
    XlCreator *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Parent( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_AcceptLabelsInFormulas( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_AcceptLabelsInFormulas( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Activate( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ActiveChart( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ActiveSheet( 
    IDispatch **RHS)
{ 
    TRACINGMETHOD; 
    CWorksheetPtr pWorksheetPtr(NULL);
    hr = m_Sheets->GetActiveSheet(&pWorksheetPtr);
    CHECKERROR;
    *RHS = pWorksheetPtr;
    (*RHS)->AddRef();
    return hr; 
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Author( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Author( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_AutoUpdateFrequency( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_AutoUpdateFrequency( 
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_AutoUpdateSaveChanges( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_AutoUpdateSaveChanges( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ChangeHistoryDuration( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_ChangeHistoryDuration( 
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_BuiltinDocumentProperties( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ChangeFileAccess( 
    XlFileAccess Mode,
    VARIANT WritePassword,
    VARIANT Notify,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ChangeLink( 
    BSTR Name,
    BSTR NewName,
    XlLinkType Type,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Charts( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Close( 
    VARIANT SaveChanges,
    VARIANT Filename,
    VARIANT RouteWorkbook,
    long lcid)
{
    TRACINGMETHOD;
    CComVariant vSaveChanges(SaveChanges);
    hr = vSaveChanges.ChangeType(VT_BOOL);
    CHECKERROR;
    if (vSaveChanges.boolVal != VARIANT_FALSE)
    {
        CComVariant vFilename(Filename);
        hr = vFilename.ChangeType(VT_BSTR);
        CHECKERROR;
        hr = m_pOOCalcDocument->GetDocument()->SaveFile(vFilename, OLESTR("MS Excel 97"));
        CHECKERROR;
    }
    hr = m_pOOCalcDocument->GetDocument()->Close();
    CHECKERROR;
    hr = m_ParentApplication->ResetWorkbooks();
    CHECKERROR
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_CodeName( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get__CodeName( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put__CodeName( 
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Colors( 
    VARIANT Index,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Colors( 
    VARIANT Index,
    long lcid,
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_CommandBars( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Comments( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Comments( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ConflictResolution( 
    XlSaveConflictResolution *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_ConflictResolution( 
    XlSaveConflictResolution RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Container( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_CreateBackup( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_CustomDocumentProperties( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Date1904( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Date1904( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::DeleteNumberFormat( 
    BSTR NumberFormat,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_DialogSheets( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_DisplayDrawingObjects( 
    long lcid,
    XlDisplayDrawingObjects *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_DisplayDrawingObjects( 
    long lcid,
    XlDisplayDrawingObjects RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ExclusiveAccess( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_FileFormat( 
    long lcid,
    XlFileFormat *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ForwardMailer( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_FullName( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_HasMailer( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_HasMailer( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_HasPassword( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_HasRoutingSlip( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_HasRoutingSlip( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_IsAddin( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_IsAddin( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Keywords( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Keywords( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::LinkInfo( 
    BSTR Name,
    XlLinkInfo LinkInfo,
    VARIANT Type,
    VARIANT EditionRef,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::LinkSources( 
    VARIANT Type,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Mailer( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::MergeWorkbook( 
    VARIANT Filename)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Modules( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_MultiUserEditing( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Name( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Names( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::NewWindow( 
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_OnSave( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_OnSave( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_OnSheetActivate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_OnSheetActivate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_OnSheetDeactivate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_OnSheetDeactivate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::OpenLinks( 
    BSTR Name,
    VARIANT ReadOnly,
    VARIANT Type,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Path( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PersonalViewListSettings( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_PersonalViewListSettings( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PersonalViewPrintSettings( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_PersonalViewPrintSettings( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::PivotCaches( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Post( 
    VARIANT DestName,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PrecisionAsDisplayed( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_PrecisionAsDisplayed( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::_PrintOut( 
    VARIANT From,
    VARIANT To,
    VARIANT Copies,
    VARIANT Preview,
    VARIANT ActivePrinter,
    VARIANT PrintToFile,
    VARIANT Collate,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::PrintPreview( 
    VARIANT EnableChanges,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::_Protect( 
    VARIANT Password,
    VARIANT Structure,
    VARIANT Windows)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ProtectSharing( 
    VARIANT Filename,
    VARIANT Password,
    VARIANT WriteResPassword,
    VARIANT ReadOnlyRecommended,
    VARIANT CreateBackup,
    VARIANT SharingPassword)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ProtectStructure( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ProtectWindows( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ReadOnly( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get__ReadOnlyRecommended( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::RefreshAll( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Reply( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ReplyAll( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::RemoveUser( 
    long Index)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_RevisionNumber( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Route( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Routed( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_RoutingSlip( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::RunAutoMacros( 
    XlRunAutoMacro Which,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Save( 
    long lcid)
{
    TRACINGMETHOD;
    // stub
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::_SaveAs( 
    VARIANT Filename,
    VARIANT FileFormat,
    VARIANT Password,
    VARIANT WriteResPassword,
    VARIANT ReadOnlyRecommended,
    VARIANT CreateBackup,
    XlSaveAsAccessMode AccessMode,
    VARIANT ConflictResolution,
    VARIANT AddToMru,
    VARIANT TextCodepage,
    VARIANT TextVisualLayout,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SaveCopyAs( 
    VARIANT Filename,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Saved( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Saved( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_SaveLinkValues( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_SaveLinkValues( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SendMail( 
    VARIANT Recipients,
    VARIANT Subject,
    VARIANT ReturnReceipt,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SendMailer( 
    VARIANT FileFormat,
    XlPriority Priority,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SetLinkOnData( 
    BSTR Name,
    VARIANT Procedure,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Sheets( 
    Sheets **ppSheets)
{ 
    TRACINGMETHOD; 
    if (ppSheets == NULL)
    {
        return E_POINTER;
    }
    *ppSheets = m_Sheets;
    (*ppSheets)->AddRef();
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ShowConflictHistory( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_ShowConflictHistory( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Styles( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Subject( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Subject( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Title( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    *RHS = ::SysAllocString(OLESTR("Workbook"));
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Title( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Unprotect( 
    VARIANT Password,
    long lcid)
{
    TRACINGMETHOD;
    CComVariant pass(Password);
    CComBSTR passStr;
    if (pass.vt == VT_EMPTY || pass.vt == VT_NULL)
    {
        passStr = OLESTR("");
    }
    else
    {
        hr = pass.ChangeType(VT_BSTR);
        CHECKERROR;
        passStr = pass.bstrVal;
    }
    hr = m_pOOCalcDocument->Unprotect(passStr);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbook::UnprotectSharing( 
    VARIANT SharingPassword)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::UpdateFromFile( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::UpdateLink( 
    VARIANT Name,
    VARIANT Type,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_UpdateRemoteReferences( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_UpdateRemoteReferences( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_UserControl( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_UserControl( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_UserStatus( 
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_CustomViews( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Windows( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Worksheets( 
    Sheets **ppSheets)
{ 
    TRACINGMETHOD; 
    return get_Sheets(ppSheets);
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_WriteReserved( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_WriteReservedBy( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Excel4IntlMacroSheets( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Excel4MacroSheets( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_TemplateRemoveExtData( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_TemplateRemoveExtData( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::HighlightChangesOptions( 
    VARIANT When,
    VARIANT Who,
    VARIANT Where)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_HighlightChangesOnScreen( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_HighlightChangesOnScreen( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_KeepChangeHistory( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_KeepChangeHistory( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ListChangesOnNewSheet( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_ListChangesOnNewSheet( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::PurgeChangeHistoryNow( 
    long Days,
    VARIANT SharingPassword)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::AcceptAllChanges( 
    VARIANT When,
    VARIANT Who,
    VARIANT Where)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::RejectAllChanges( 
    VARIANT When,
    VARIANT Who,
    VARIANT Where)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::PivotTableWizard( 
    VARIANT SourceType,
    VARIANT SourceData,
    VARIANT TableDestination,
    VARIANT TableName,
    VARIANT RowGrand,
    VARIANT ColumnGrand,
    VARIANT SaveData,
    VARIANT HasAutoFormat,
    VARIANT AutoPage,
    VARIANT Reserved,
    VARIANT BackgroundQuery,
    VARIANT OptimizeCache,
    VARIANT PageFieldOrder,
    VARIANT PageFieldWrapCount,
    VARIANT ReadData,
    VARIANT Connection,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ResetColors( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_VBProject( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::FollowHyperlink( 
    BSTR Address,
    VARIANT SubAddress,
    VARIANT NewWindow,
    VARIANT AddHistory,
    VARIANT ExtraInfo,
    VARIANT Method,
    VARIANT HeaderInfo)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::AddToFavorites( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_IsInplace( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::PrintOut( 
    VARIANT From,
    VARIANT To,
    VARIANT Copies,
    VARIANT Preview,
    VARIANT ActivePrinter,
    VARIANT PrintToFile,
    VARIANT Collate,
    VARIANT PrToFileName,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::WebPagePreview( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PublishObjects( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_WebOptions( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ReloadAs( 
    MsoEncoding Encoding)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_HTMLProject( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_EnvelopeVisible( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_EnvelopeVisible( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_CalculationVersion( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Dummy17( 
    long calcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::sblt( 
    BSTR s)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_VBASigned( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ShowPivotTableFieldList( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_ShowPivotTableFieldList( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_UpdateLinks( 
    XlUpdateLinks *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_UpdateLinks( 
    XlUpdateLinks RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::BreakLink( 
    BSTR Name,
    XlLinkType Type)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Dummy16( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SaveAs( 
    VARIANT Filename,
    VARIANT FileFormat,
    VARIANT Password,
    VARIANT WriteResPassword,
    VARIANT ReadOnlyRecommended,
    VARIANT CreateBackup,
    XlSaveAsAccessMode AccessMode,
    VARIANT ConflictResolution,
    VARIANT AddToMru,
    VARIANT TextCodepage,
    VARIANT TextVisualLayout,
    VARIANT Local,
    long lcid)
{
    TRACINGMETHOD;
    CComVariant vFilename(Filename);
    hr = vFilename.ChangeType(VT_BSTR);
    CHECKERROR;
    std::wstring sPath = vFilename.bstrVal;
    std::wstring sURL;
    COOApplication::ConvertPathToUrl(sPath, sURL);
    hr = m_pOOCalcDocument->GetDocument()->SaveFile(CComVariant(sURL.c_str()), OLESTR("MS Excel 97"));
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_EnableAutoRecover( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_EnableAutoRecover( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_RemovePersonalInformation( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_RemovePersonalInformation( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_FullNameURLEncoded( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::CheckIn( 
    VARIANT SaveChanges,
    VARIANT Comments,
    VARIANT MakePublic)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::CanCheckIn( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SendForReview( 
    VARIANT Recipients,
    VARIANT Subject,
    VARIANT ShowMessage,
    VARIANT IncludeAttachment)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ReplyWithChanges( 
    VARIANT ShowMessage)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::EndReview( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Password( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_Password( 
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_WritePassword( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_WritePassword( 
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PasswordEncryptionProvider( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PasswordEncryptionAlgorithm( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PasswordEncryptionKeyLength( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SetPasswordEncryptionOptions( 
    VARIANT PasswordEncryptionProvider,
    VARIANT PasswordEncryptionAlgorithm,
    VARIANT PasswordEncryptionKeyLength,
    VARIANT PasswordEncryptionFileProperties)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_PasswordEncryptionFileProperties( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_ReadOnlyRecommended( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_ReadOnlyRecommended( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::Protect( 
    VARIANT Password,
    VARIANT Structure,
    VARIANT Windows)
{
    TRACINGMETHOD;
    CComVariant pass(Password);
    CComBSTR passStr;
    if (pass.vt == VT_EMPTY || pass.vt == VT_NULL)
    {
        passStr = OLESTR("");
    }
    else
    {
        hr = pass.ChangeType(VT_BSTR);
        CHECKERROR;
        passStr = pass.bstrVal;
    }
    hr = m_pOOCalcDocument->Protect(passStr);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_SmartTagOptions( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::RecheckSmartTags( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Permission( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_SharedWorkspace( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_Sync( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SendFaxOverInternet( 
    VARIANT Recipients,
    VARIANT Subject,
    VARIANT ShowMessage)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_XmlNamespaces( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_XmlMaps( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::XmlImport( 
    BSTR Url,
    IDispatch **ImportMap,
    VARIANT Overwrite,
    VARIANT Destination,
    XlXmlImportResult *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_SmartDocument( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_DocumentLibraryVersions( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_InactiveListBorderVisible( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_InactiveListBorderVisible( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::get_DisplayInkComments( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::put_DisplayInkComments( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::XmlImportXml( 
    BSTR Data,
    IDispatch **ImportMap,
    VARIANT Overwrite,
    VARIANT Destination,
    XlXmlImportResult *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::SaveAsXMLData( 
    BSTR Filename,
    IDispatch *Map)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorkbook::ToggleFormsDesign( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}


