
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "stdafx.h"
#include "OOApplication.h"

using namespace AutomateIT::Utils;
using namespace AutomateIT::OpenOfficeWrapper;

COOApplication::COOApplication():
    m_pdDesktop(NULL),
    m_pdOOApp(NULL),
    m_pdDatabaseContext(NULL),
    m_pdActiveConnection(NULL)
{
	CreateApplication();
}

COOApplication::~COOApplication()
{
}

IDispatchPtr COOApplication::GetOOApp()
{
	return m_pdOOApp;
}

IDispatchPtr COOApplication::GetOODesktop()
{
	return m_pdDesktop;
}

HRESULT COOApplication::CreateApplication()
{
	CLSID clsid;  
	HRESULT hr = CLSIDFromProgID(L"com.sun.star.ServiceManager", &clsid); 
    CHECKERROR;
    // Start server and get IDispatch... 

    IDispatch* pdOOApp(NULL);
    IDispatchPtr pdOOAppPtr(NULL);
    IDispatchPtr pdDesktopPtr(NULL);

    hr = CoCreateInstance(clsid, NULL, CLSCTX_LOCAL_SERVER, IID_IDispatch, (void **)&pdOOApp);
    CHECKERROR;

    pdOOAppPtr = pdOOApp;

    CComVariant result;
    hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, pdOOAppPtr, L"createInstance", 1, CComVariant(L"com.sun.star.frame.Desktop")); 
    CHECKERROR;

    pdDesktopPtr = result.pdispVal; 

    m_pdOOApp = pdOOAppPtr;
    m_pdDesktop = pdDesktopPtr;

    return S_OK;
}


HRESULT COOApplication::GetDispatchPropertyValue(IDispatch** pIDispatch)
{
    CComVariant res;
    CComVariant cvStr(L"com.sun.star.beans.PropertyValue");
    HRESULT hres = AutoWrapper::Execute(DISPATCH_METHOD, &res, m_pdOOApp, L"Bridge_GetStruct", 1, cvStr);
    if (hres == S_OK) 
    {
        *pIDispatch = res.pdispVal;
        (*pIDispatch)->AddRef();
    }
    else
	{
        *pIDispatch = NULL;
	}
    return hres;
}

HRESULT COOApplication::CreateBlankDocument(const std::wstring& sDocumentType, COODocument** pDocument, BOOL bShowed)
{
    TRACINGMETHOD;

    CComVariant params[4];
    params[0] = sDocumentType.c_str();  // Type of created document
    params[1] = L"_blank";              // Template
    params[2] = 1;                      // Another params count

    CPropertyValue PropValue(m_pdOOApp);
    VARIANT pp;
    VariantInit (&pp);
    pp.vt = VT_BOOL;
    if (bShowed)
    {
        pp.boolVal = (VARIANT_FALSE);
    }
    else
    {
        pp.boolVal = (VARIANT_TRUE);
    }
    
	PropValue.SetPropertyValue(CComVariant(L"Hidden"), pp);

    SAFEARRAY FAR* pPropVals = SafeArrayCreateVector(VT_DISPATCH, 0, 1);
    long ix = 0; 
    hr = SafeArrayPutElement(pPropVals, &ix, PropValue.GetValueDispatch()); 
    CHECKERROR;

    params[3].vt = VT_DISPATCH | VT_ARRAY;
    params[3].parray = pPropVals;

	CComVariant ResultDoc;
	hr = AutoWrapper::Execute(DISPATCH_METHOD, &ResultDoc, m_pdDesktop, L"loadComponentFromURL", 4, params[0], params[1], params[2], params[3]); 
    CHECKERROR;

    IDispatchPtr pdDoc = ResultDoc.pdispVal; 
    if (pDocument != NULL) 
    {
        COODocumentObject* pOODoc(NULL);
        COODocumentPtr pOODocPtr(NULL);
        hr = COODocumentObject::CreateInstance(&pOODoc);
        CHECKERROR;
        
        pOODocPtr = pOODoc;
        hr = pOODocPtr->Initialize(this, pdDoc);
        CHECKERROR;

        *pDocument = pOODocPtr;
        (*pDocument)->AddRef();
    }
    return hr;
}

HRESULT COOApplication::FileOpen(CComVariant vName, COODocument** pDocument, BOOL bShowed)
{
    HRESULT hr = S_OK;

    CComVariant res;    
    IDispatch *dpv = NULL;
    CComVariant p1 (L"Hidden");

    CComVariant ResultDoc;
    CComVariant params[4]; 
    VariantInit(&params[0]);            //private:factory/scalc
    params[0] = vName;                  // Type of created document
    params[1] = L"_blank";              // Template
    params[2] = 1;                      // Another params count
    if (!bShowed)
    {
        CPropertyValue PropValue(m_pdOOApp);
        CComVariant pp;

        pp.vt = VT_BOOL;
        pp.boolVal = (VARIANT_TRUE);

        PropValue.SetPropertyValue(p1, pp);

        SAFEARRAY FAR* pPropVals = SafeArrayCreateVector( VT_DISPATCH, 0, 1 );
        long ix = 0; 
        hr = SafeArrayPutElement( pPropVals, &ix, PropValue.GetValueDispatch() ); 
        CHECKERROR;

        VariantInit(&params[3]); 
        params[3].vt = VT_DISPATCH | VT_ARRAY;
        params[3].parray = pPropVals;
    }
    else
    {
        params[3] = NULL;
        params[3].vt =  VT_ARRAY | VT_VARIANT; 
    }
	hr = AutoWrapper::Execute(DISPATCH_METHOD, &ResultDoc, m_pdDesktop, L"loadComponentFromURL", 4, params[0], params[1], params[2], params[3]);  
    CHECKERROR;

    IDispatchPtr pdDoc = ResultDoc.pdispVal; 
    if (pDocument != NULL) 
    {
        COODocumentObject* pOODoc(NULL);
        COODocumentPtr pOODocPtr(NULL);
        hr = COODocumentObject::CreateInstance(&pOODoc);
        CHECKERROR;

        pOODocPtr = pOODoc;
        hr = pOODocPtr->Initialize(this, pdDoc);
        CHECKERROR;

        *pDocument = pOODocPtr;
        (*pDocument)->AddRef();
    }
    return hr;
}

/*
Convert Windows path to URL 
*/
void COOApplication::ConvertPathToUrl(const std::wstring &sPath, std::wstring &sURL)
{
    std::wstring sSource = sPath;
    std::wstring::iterator iEnd = sSource.end();
    std::wstring::iterator iCursor = sSource.begin();
    for (;iCursor != iEnd; iCursor++)
    {
        if (*iCursor == L'\\')
        {
            *iCursor = L'/';
        }
    }
    sURL = L"file:///" + sSource;
}


HRESULT COOApplication::SetPropertyValue(IDispatch* pdProperty, const std::wstring& sName, const CComVariant& rValue)
{
    if (pdProperty == NULL)
	{
        return E_INVALIDARG;
	}
    CComVariant p1 = sName.c_str();
    CComVariant res;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &res, pdProperty, L"Name", 1, p1);
    CHECKERROR;
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &res, pdProperty, L"Value", 1, rValue);
    CHECKERROR;
    return hr;
}


HRESULT COOApplication::SetVisible(VARIANT_BOOL visible)
{
    CPropertyValue propValue(m_pdOOApp);
    return propValue.SetPropertyValue(CComVariant(L"Hidden"), CComVariant(visible));
}

HRESULT COOApplication::GetVisible(VARIANT_BOOL* visible)
{
    CPropertyValue propValue(m_pdOOApp);
    CComVariant pp(false);

    HRESULT hr = propValue.GetPropertyValue(CComVariant(L"Hidden"), &pp);
    if (SUCCEEDED(hr))
    {
        *visible = !pp.boolVal;
    }
    return hr;
}

HRESULT COOApplication::CreateDispatchHelper( COODispatchHelper** ppDispatchHelper )
{
    IDispatchPtr pDispatch(NULL);
    
    CComVariant param(OLESTR("com.sun.star.frame.DispatchHelper"));
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdOOApp, L"createInstance", 1, param);
    CHECKERROR;

    pDispatch = result.pdispVal;

    COODispatchHelperObject* pDispatchHelperObject(NULL);
    hr = COODispatchHelperObject::CreateInstance(&pDispatchHelperObject);
    CHECKERROR;
    
    COODispatchHelperPtr pDispatchHelper(pDispatchHelperObject);
    hr = pDispatchHelper->Initialize(pDispatch);
    CHECKERROR;

    (*ppDispatchHelper) = pDispatchHelper;
    (*ppDispatchHelper)->AddRef();

    return hr;
}


HRESULT COOApplication::get_Components( COOComponents** ppComponents )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdDesktop, L"Components", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COOComponentsObject* pComponentsObject(NULL); 
    hr = COOComponentsObject::CreateInstance(&pComponentsObject);
    CHECKERROR;

    COOComponentsPtr pComponents(pComponentsObject); 
    hr = pComponents->Initialize(dispatch);
    CHECKERROR;

    (*ppComponents) = pComponents;
    (*ppComponents)->AddRef();

    return hr;
}
