
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _OO_APPLICATION_H_
#define _OO_APPLICATION_H_

#include <string>
#include <vector>

#include "OODocument.h"
#include "IDispatchWrapper.h"

/*
	COOAplication is a bridge to the Open Office. It provides a functionality similar to Word Application class
	by utilizing OpenOffice API.
*/

namespace AutomateIT
{
	namespace OpenOfficeWrapper
	{
        class COODispatchHelper;
        class COOComponents;

        class ATL_NO_VTABLE COOApplication : 
            public CComObjectRootEx<CComSingleThreadModel>,
            public IUnknown
		{
		public:

            BEGIN_COM_MAP(COOApplication)
                COM_INTERFACE_ENTRY(IUnknown)
            END_COM_MAP( )

			COOApplication();
			virtual ~COOApplication();

			IDispatchPtr GetOOApp();	
			IDispatchPtr GetOODesktop();
			
            HRESULT GetDispatchPropertyValue(IDispatch** pIDispatch);
			HRESULT SetPropertyValue(IDispatch* pdProperty, const std::wstring& sName, const CComVariant& rValue);
			HRESULT CreateBlankDocument(const std::wstring& sDocumentType, COODocument** pDocument, BOOL bShowed);
			HRESULT FileOpen(CComVariant vName, COODocument** pDocument, BOOL bShowed);

            static void ConvertPathToUrl(const std::wstring &sPath, std::wstring& sURL);

            HRESULT SetVisible(VARIANT_BOOL visible);
            HRESULT GetVisible(VARIANT_BOOL* visible);

            HRESULT CreateDispatchHelper(COODispatchHelper** ppDispatchHelper);

            HRESULT get_Components(COOComponents** ppComponents);

		private:
			HRESULT CreateApplication();
			HRESULT DestroyApplication();

		private:
			IDispatchPtr m_pdOOApp;
			IDispatchPtr m_pdDesktop;
			IDispatchPtr m_pdDatabaseContext;
			IDispatchPtr m_pdActiveConnection;
		};

        typedef CComPtr<COOApplication> COOApplicationPtr;
        typedef CComObject<COOApplication> COOApplicationObject;
	}
}

#endif //_OO_APPLICATION_H_