/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "OORange.h"

using namespace AutomateIT::Utils;
using namespace AutomateIT::OpenOfficeWrapper;

COORange::COORange()
{
}

COORange::~COORange()
{
}

HRESULT COORange::Initialize( IDispatchPtr dispatch )
{
    m_pdRange = dispatch;
    return S_OK;
}

HRESULT COORange::GetCellRangeByPosition( long lLeft, long lTop, long lRight, long lBottom, COORange** ppOORange )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getCellRangeByPosition", 4, CComVariant(lLeft), CComVariant(lTop), CComVariant(lRight), CComVariant(lBottom));
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COORangeObject* pRange(NULL); 
    hr = COORangeObject::CreateInstance(&pRange);
    CHECKERROR;

    COORangePtr pRangePtr(pRange); 
    hr = pRangePtr->Initialize(dispatch);
    CHECKERROR;

    (*ppOORange) = pRangePtr;
    (*ppOORange)->AddRef();

    return hr;

}

HRESULT COORange::GetCellByPosition( long lLeft, long lTop, COOCell** ppOOCell )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getCellByPosition", 2, CComVariant(lLeft), CComVariant(lTop));
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COOCellObject* pCell(NULL); 
    hr = COOCellObject::CreateInstance(&pCell);
    CHECKERROR;

    COOCellPtr pCellPtr(pCell); 
    hr = pCellPtr->Initialize(dispatch);
    CHECKERROR;

    (*ppOOCell) = pCellPtr;
    (*ppOOCell)->AddRef();

    return hr;

}

HRESULT COORange::GetCellRangeByName( BSTR bstrName, COORange** ppOORange )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getCellRangeByName", 1, CComVariant(bstrName));
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COORangeObject* pRange(NULL); 
    hr = COORangeObject::CreateInstance(&pRange);
    CHECKERROR;

    COORangePtr pRangePtr(pRange); 
    hr = pRangePtr->Initialize(dispatch);
    CHECKERROR;

    (*ppOORange) = pRangePtr;
    (*ppOORange)->AddRef();

    return hr;
}

HRESULT COORange::get_Spreadsheet( COORange** ppSpreadsheet )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"Spreadsheet", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COORangeObject* pRange(NULL); 
    hr = COORangeObject::CreateInstance(&pRange);
    CHECKERROR;

    COORangePtr pRangePtr(pRange); 
    hr = pRangePtr->Initialize(dispatch);
    CHECKERROR;

    (*ppSpreadsheet) = pRangePtr;
    (*ppSpreadsheet)->AddRef();

    return hr;
}

HRESULT COORange::GetRangeAddress( COORangeAddress** ppOORangeAddress )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getRangeAddress", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COORangeAddressObject* pRangeAddress(NULL); 
    hr = COORangeAddressObject::CreateInstance(&pRangeAddress);
    CHECKERROR;

    COORangeAddressPtr pRangeAddressPtr(pRangeAddress); 
    hr = pRangeAddressPtr->Initialize(dispatch);
    CHECKERROR;

    (*ppOORangeAddress) = pRangeAddressPtr;
    (*ppOORangeAddress)->AddRef();

    return hr;
}

HRESULT COORange::GetColumns( COOColumns** ppOOColumns )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getColumns", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COOColumnsObject* pColumns(NULL); 
    hr = COOColumnsObject::CreateInstance(&pColumns);
    CHECKERROR;

    COOColumnsPtr pColumnsPtr(pColumns); 
    hr = pColumnsPtr->Initialize(dispatch);
    CHECKERROR;

    (*ppOOColumns) = pColumnsPtr;
    (*ppOOColumns)->AddRef();

    return hr;
}

HRESULT COORange::GetRows( COORows** ppOORows )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getRows", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COORowsObject* pRows(NULL); 
    hr = COORowsObject::CreateInstance(&pRows);
    CHECKERROR;

    COORowsPtr pRowsPtr(pRows); 
    hr = pRowsPtr->Initialize(dispatch);
    CHECKERROR;

    (*ppOORows) = pRowsPtr;
    (*ppOORows)->AddRef();

    return hr;
}

HRESULT COORange::get_CharLocale( COOCharLocale** ppCharLocale )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharLocale", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COOCharLocaleObject* pCharLocale(NULL); 
    hr = COOCharLocaleObject::CreateInstance(&pCharLocale);
    CHECKERROR;

    COOCharLocalePtr pCharLocalePtr(pCharLocale); 
    hr = pCharLocalePtr->Initialize(dispatch);
    CHECKERROR;

    (*ppCharLocale) = pCharLocalePtr;
    (*ppCharLocale)->AddRef();

    return hr;
}

HRESULT COORange::get_TableBorder( COOTableBorder** ppTableBorder )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"TableBorder", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COOTableBorderObject* pTableBorder(NULL); 
    hr = COOTableBorderObject::CreateInstance(&pTableBorder);
    CHECKERROR;

    COOTableBorderPtr pTableBorderPtr(pTableBorder); 
    hr = pTableBorderPtr->Initialize(dispatch);
    CHECKERROR;

    (*ppTableBorder) = pTableBorderPtr;
    (*ppTableBorder)->AddRef();

    return hr;
}

HRESULT COORange::put_TableBorder( COOTableBorder* pTableBorder )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"TableBorder", 1, CComVariant(pTableBorder->m_pdTableBorder.p));
    CHECKERROR;
    return hr;
}


HRESULT COORange::GetName( BSTR* pbstrName )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getName", 0);
    CHECKERROR;
    *pbstrName = result.bstrVal;
    return hr;
}

HRESULT COORange::SetName( BSTR bstrName )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"setName", 1, CComVariant(bstrName));
    CHECKERROR;
    return hr;

}

HRESULT COORange::get_IsTextWrapped( bool* pbWrapped )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"IsTextWrapped", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbWrapped) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_IsTextWrapped( bool bWrapped )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"IsTextWrapped", 1, CComVariant(bWrapped));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_CharWeight( float* pfWeight )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharWeight", 0);
    CHECKERROR;
    (*pfWeight) = result.fltVal;
    return hr;
}

HRESULT COORange::put_CharWeight( float fWeight )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharWeight", 1, CComVariant(fWeight));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_CharColor( int* piColor )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharColor", 0);
    CHECKERROR;
    (*piColor) = result.intVal;
    return hr;
}

HRESULT COORange::put_CharColor( int iColor )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharColor", 1, CComVariant(iColor));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_CharPosture( int* piPosture )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharPosture", 0);
    CHECKERROR;
    (*piPosture) = result.intVal;
    return hr;
}

HRESULT COORange::put_CharPosture( int iPosture )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharPosture", 1, CComVariant(iPosture));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_CharHeight( int* piHeight )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharHeight", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_I4);
    CHECKERROR;
    (*piHeight) = result.intVal;
    return hr;
}

HRESULT COORange::put_CharHeight( int iHeight )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharHeight", 1, CComVariant(iHeight));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_CharUnderline( bool* pbUnderline )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharUnderline", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbUnderline) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_CharUnderline( bool bUnderline )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharUnderline", 1, CComVariant(bUnderline));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_CharShadowed( bool* pbCharShadowed )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharShadowed", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbCharShadowed) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_CharShadowed( bool bCharShadowed )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharShadowed", 1, CComVariant(bCharShadowed));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_CharStrikeout( bool* pbCharStrikeout )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharStrikeout", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbCharStrikeout) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_CharStrikeout( bool bCharStrikeout )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharStrikeout", 1, CComVariant(bCharStrikeout));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_IsCellBackgroundTransparent( bool* pbValue )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"IsCellBackgroundTransparent", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbValue) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_IsCellBackgroundTransparent( bool bValue )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"IsCellBackgroundTransparent", 1, CComVariant(bValue));
    CHECKERROR;
    return hr;

}

HRESULT COORange::get_cellBackColor( int* piColor )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"cellBackColor", 0);
    CHECKERROR;
    (*piColor) = result.intVal;
    return hr;

}

HRESULT COORange::put_cellBackColor( int iColor )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"cellBackColor", 1, CComVariant(iColor));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_VertJustify( int* piValue )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"VertJustify", 0);
    CHECKERROR;
    (*piValue) = result.intVal;
    return hr;
}

HRESULT COORange::put_VertJustify( int iValue )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"VertJustify", 1, CComVariant(iValue));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_HoriJustify( int* piValue )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"HoriJustify", 0);
    CHECKERROR;
    (*piValue) = result.intVal;
    return hr;
}

HRESULT COORange::put_HoriJustify( int iValue )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"HoriJustify", 1, CComVariant(iValue));
    CHECKERROR;
    return hr;
}


HRESULT COORange::get_NumberFormat( int* piNumberFormat )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"NumberFormat", 0);
    CHECKERROR;
    (*piNumberFormat) = result.intVal;
    return hr;
}

HRESULT COORange::put_NumberFormat( int iNumberFormat )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"NumberFormat", 1, CComVariant(iNumberFormat));
    CHECKERROR;
    return hr;
}



HRESULT COORange::get_CharFontName( BSTR* pbstrCharFontName )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CharFontName", 0);
    CHECKERROR;
    *pbstrCharFontName = result.bstrVal;
    return hr;
}


HRESULT COORange::put_CharFontName( BSTR bstrCharFontName )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CharFontName", 1, CComVariant(bstrCharFontName));
    CHECKERROR;
    return hr;
}



HRESULT COORange::Protect(BSTR bstrPassword)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"protect", 1, CComVariant(bstrPassword));
    CHECKERROR;
    return hr;
}

HRESULT COORange::Unprotect(BSTR bstrPassword)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"unprotect", 1, CComVariant(bstrPassword));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_PageStyle(BSTR* pbstrPageStyle)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"PageStyle", 0);
    CHECKERROR;
    *pbstrPageStyle = result.bstrVal;
    return hr;
}

HRESULT COORange::get_Height( short* psHeight )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"Size", 0);
    CHECKERROR;
    IDispatchPtr pSize = result.pdispVal;
    
    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pSize, L"Height", 0);
    CHECKERROR;

    (*psHeight) = result.iVal;
    return hr;
}

HRESULT COORange::get_Width( short* psWidth )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"Size", 0);
    CHECKERROR;
    IDispatchPtr pSize = result.pdispVal;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pSize, L"Width", 0);
    CHECKERROR;

    (*psWidth) = result.iVal;
    return hr;
}

HRESULT COORange::get_ShrinkToFit( bool* pbShrinkToFit )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"ShrinkToFit", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbShrinkToFit) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_ShrinkToFit( bool bShrinkToFit )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"ShrinkToFit", 1, CComVariant(bShrinkToFit));
    CHECKERROR;
    return hr;
}

HRESULT COORange::ClearContents( int iValue )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"clearContents", 1, CComVariant(iValue)); 
    CHECKERROR;
    return hr;
}

HRESULT COORange::GetIsMerged(bool* pbIsMerged)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"getIsMerged", 0);
    CHECKERROR;
    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbIsMerged) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::Merge(bool bValue)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pdRange, L"Merge", 1, CComVariant(bValue));
    CHECKERROR;
    return hr;
}

HRESULT COORange::get_IsHidden(bool* pbIsHidden)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CellProtection", 0);
    CHECKERROR;

    IDispatchPtr pdCellProtection = result.pdispVal;

    result.Clear();

    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pdCellProtection, L"IsHidden", 0);
    CHECKERROR;

    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbIsHidden) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_IsHidden(bool bIsHidden)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CellProtection", 0);
    CHECKERROR;

    IDispatchPtr pdCellProtection = result.pdispVal;

    result.Clear();

    hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, pdCellProtection, L"IsHidden", 1, CComVariant(bIsHidden));
    CHECKERROR;

    result.Clear();

    hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CellProtection", 1, CComVariant(pdCellProtection.p));
    CHECKERROR;

    return hr;
}

HRESULT COORange::get_IsLocked(bool* pbIsLocked)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CellProtection", 0);
    CHECKERROR;

    IDispatchPtr pdCellProtection = result.pdispVal;

    result.Clear();

    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pdCellProtection, L"IsLocked", 0);
    CHECKERROR;

    hr = result.ChangeType(VT_BOOL);
    CHECKERROR;
    (*pbIsLocked) = (result.boolVal != VARIANT_FALSE);
    return hr;
}

HRESULT COORange::put_IsLocked(bool bIsLocked)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pdRange, L"CellProtection", 0);
    CHECKERROR;

    IDispatchPtr pdCellProtection = result.pdispVal;

    result.Clear();

    hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, pdCellProtection, L"IsLocked", 1, CComVariant(bIsLocked));
    CHECKERROR;

    result.Clear();

    hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, m_pdRange, L"CellProtection", 1, CComVariant(pdCellProtection.p));
    CHECKERROR;

    return hr;
}
