
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _OO_RANGE_H_
#define _OO_RANGE_H_

#pragma once

#include "IDispatchWrapper.h"

namespace AutomateIT
{
    namespace OpenOfficeWrapper
    {
        class COORangeAddress;
        class COOColumns;
        class COORows;
        class COOCharLocale;
        class COOCell;
        class COOTableBorder;

        class ATL_NO_VTABLE COORange : 
            public CComObjectRootEx<CComSingleThreadModel>,
            public IUnknown
        {
        public:
            enum CellFlags
            {
                VALUE 	    = 1,
                DATETIME 	= 2,
                STRING  	= 4,
                ANNOTATION 	= 8,
                FORMULA 	= 16,
                HARDATTR 	= 32,
                STYLES 	    = 64,
                OBJECTS 	= 128,
                EDITATTR 	= 256,
                FORMATTED 	= 512
            };

        public:

            friend class COOCalcDocument;

            BEGIN_COM_MAP(COORange)
                COM_INTERFACE_ENTRY(IUnknown)
            END_COM_MAP( )

            COORange();
            virtual ~COORange();

            HRESULT Initialize(IDispatchPtr dispatch);

            HRESULT GetName(BSTR* pbstrName);
            HRESULT SetName(BSTR bstrName);

            HRESULT GetCellRangeByPosition(long lLeft, long lTop, long lRight, long lBottom, COORange** ppOORange);
            HRESULT GetCellByPosition(long lLeft, long lTop, COOCell** ppOOCell);
            HRESULT GetCellRangeByName(BSTR bstrName, COORange** ppOORange);

            HRESULT GetRangeAddress(COORangeAddress** ppOORangeAddress);
            HRESULT GetColumns(COOColumns** ppOOColumns);
            HRESULT GetRows(COORows** ppOORows);

            HRESULT get_IsTextWrapped(bool* pbWrapped);
            HRESULT put_IsTextWrapped(bool bWrapped);

            HRESULT get_CharWeight(float* pfWeight);
            HRESULT put_CharWeight(float fWeight);

            HRESULT get_CharHeight(int* piHeight);
            HRESULT put_CharHeight(int iHeight);

            HRESULT get_CharColor(int* piColor);
            HRESULT put_CharColor(int iColor);

            HRESULT get_CharPosture(int* piPosture);
            HRESULT put_CharPosture(int iPosture);

            HRESULT get_CharUnderline(bool* pbUnderline);
            HRESULT put_CharUnderline(bool bUnderline);

            HRESULT get_CharShadowed(bool* pbCharShadowed);
            HRESULT put_CharShadowed(bool bCharShadowed);

            HRESULT get_CharStrikeout(bool* pbCharStrikeout);
            HRESULT put_CharStrikeout(bool bCharStrikeout);

            HRESULT get_IsCellBackgroundTransparent(bool* pbValue);
            HRESULT put_IsCellBackgroundTransparent(bool bValue);

            HRESULT get_cellBackColor(int* piColor);
            HRESULT put_cellBackColor(int iColor);

            HRESULT get_VertJustify(int* piValue);
            HRESULT put_VertJustify(int iValue);

            HRESULT get_HoriJustify(int* piValue);
            HRESULT put_HoriJustify(int iValue);

            HRESULT get_NumberFormat(int* piNumberFormat);
            HRESULT put_NumberFormat(int iNumberFormat);

            HRESULT get_CharFontName(BSTR* pbstrCharFontName);
            HRESULT put_CharFontName(BSTR bstrCharFontName);

            HRESULT get_PageStyle(BSTR* pbstrPageStyle);

            HRESULT get_Spreadsheet(COORange** ppSpreadsheet);
            
            HRESULT get_CharLocale(COOCharLocale** ppCharLocale);

            HRESULT get_TableBorder(COOTableBorder** ppTableBorder);
            HRESULT put_TableBorder(COOTableBorder* pTableBorder);

            HRESULT get_Height(short* psHeight);

            HRESULT get_Width(short* psWidth);

            HRESULT get_ShrinkToFit(bool* pbShrinkToFit);
            HRESULT put_ShrinkToFit(bool bShrinkToFit);

            HRESULT Protect(BSTR bstrPassword);
            HRESULT Unprotect(BSTR bstrPassword);

            HRESULT GetIsMerged(bool* pbIsMerged);

            HRESULT ClearContents(int iValue);

            HRESULT Merge(bool bValue);

            HRESULT get_IsHidden(bool* pbIsHidden);
            HRESULT put_IsHidden(bool bIsHidden);

            HRESULT get_IsLocked(bool* pbIsLocked);
            HRESULT put_IsLocked(bool bIsLocked);

        private:

            IDispatchPtr m_pdRange;

        };


        typedef CComPtr<COORange> COORangePtr;
        typedef CComObject<COORange> COORangeObject;

    }
}

#endif // _OO_RANGE_H_

