
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#pragma once

#include "WordClone.h"

#include "OOApplication.h"
#include "OODocument.h"
#include "OOWriterDocument.h"

namespace AutomateIT
{
	namespace WordClone
	{
		struct CMailMergeParams
		{
			CComVariant Name;
			CComVariant ConfirmConversions;
			CComVariant ReadOnly;
			CComVariant LinkToSource;
			CComVariant AddToMru;
			CComVariant PasswordDoc;
			CComVariant PasswordDot;
			CComVariant Revert;
			CComVariant Connection;
		};

		/*
		CWordBasic class provides functionality of the MS Office WordBasic by implementing IWordBasic interface.
		*/
		class ATL_NO_VTABLE CWordBasic :
			public CComObjectRootEx<CComSingleThreadModel>,
			public CComCoClass<CWordBasic, &CLSID_WordBasic>,
			public IDispatchImpl<IWordBasic, &IID_IWordBasic, &LIBID_WordCloneLib, /*wMajor =*/ 1, /*wMinor =*/ 0>
		{
		public:
			CWordBasic();
			~CWordBasic();

			DECLARE_CLASSFACTORY_SINGLETON(CWordBasic)

			DECLARE_REGISTRY_RESOURCEID(IDR_WORDBASIC)

			DECLARE_NOT_AGGREGATABLE(CWordBasic)

			BEGIN_COM_MAP(CWordBasic)
				COM_INTERFACE_ENTRY(IWordBasic)
				COM_INTERFACE_ENTRY(IDispatch)
			END_COM_MAP()

		public:


			static COOApplication s_pOOApp;
			COOWriterDocument *m_pOOWDoc;

			// IWordBasic methods

			STDMETHOD(FileOpen)( 
				VARIANT Name,
				VARIANT ConfirmConversions,
				VARIANT ReadOnly,
				VARIANT AddToMru,
				VARIANT PasswordDoc,
				VARIANT PasswordDot,
				VARIANT Revert,
				VARIANT WritePasswordDoc,
				VARIANT WritePasswordDot
				);

			STDMETHOD(MailMergeOpenDataSource)( 
				VARIANT Name,
				VARIANT ConfirmConversions,
				VARIANT ReadOnly,
				VARIANT LinkToSource,
				VARIANT AddToMru,
				VARIANT PasswordDoc,
				VARIANT PasswordDot,
				VARIANT Revert,
				VARIANT WritePasswordDoc,
				VARIANT WritePasswordDot,
				VARIANT Connection,
				VARIANT SQLStatement,
				VARIANT SQLStatement1
				);

			STDMETHOD(ToolsOptionsEdit)( 
				VARIANT ReplaceSelection,
				VARIANT DragAndDrop,
				VARIANT AutoWordSelection,
				VARIANT InsForPaste,
				VARIANT Overtype,
				VARIANT SmartCutPaste,
				VARIANT AllowAccentedUppercase,
				VARIANT PictureEditor,
				VARIANT TabIndent
				);

			STDMETHOD(FileNewDefault)();

			STDMETHOD(FormFieldOptions)( 
				VARIANT Entry,
				VARIANT Exit,
				VARIANT Name,
				VARIANT Enable,
				VARIANT TextType,
				VARIANT TextWidth,
				VARIANT TextDefault,
				VARIANT TextFormat,
				VARIANT CheckSize,
				VARIANT CheckWidth,
				VARIANT CheckDefault,
				VARIANT Type,
				VARIANT OwnHelp,
				VARIANT HelpText,
				VARIANT OwnStat,
				VARIANT StatText
				);

			STDMETHOD(InsertFormField)( 
				VARIANT Entry,
				VARIANT Exit,
				VARIANT Name,
				VARIANT Enable,
				VARIANT TextType,
				VARIANT TextWidth,
				VARIANT TextDefault,
				VARIANT TextFormat,
				VARIANT CheckSize,
				VARIANT CheckWidth,
				VARIANT CheckDefault,
				VARIANT Type,
				VARIANT OwnHelp,
				VARIANT HelpText,
				VARIANT OwnStat,
				VARIANT StatText
				);

			STDMETHOD(MailMergeToDoc)();

			STDMETHOD(ShowAll)(VARIANT On, short *pResult);

			STDMETHOD(AppActivate)(BSTR WindowName, VARIANT Immediate);

			STDMETHOD(NextField)(short *pResult);

			STDMETHOD(AppShow)(VARIANT WindowName);

			STDMETHOD(GetFormResultStr)(BSTR BookmarkName, VARIANT *pResult);

			STDMETHOD(SetFormResult)( 
				BSTR    BookmarkName,
				VARIANT Result,
				VARIANT DefaultResult
				);

			STDMETHOD(GetSelStartPos)(long *pSelStartPos);

			STDMETHOD(SetSelRange)(long Pos1, long Pos2);

			STDMETHOD_(void, DocClose) (VARIANT Save);
		};

		OBJECT_ENTRY_AUTO(__uuidof(WordBasic), CWordBasic)
	}
}


